// WidcommStack.cpp: implementation of the CWidcommStack class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "wcbts.h"
#include "WidcommStack.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


CWidcommStack::CWidcommStack()
:	m_deviceResponded	(NULL),
	m_inquiryComplete	(NULL),
	m_discoveryComplete	(NULL),
	m_hWnd				(NULL),
	m_nMsg				(0)
{
	memset(m_bdAddr, 0, sizeof(m_bdAddr));
}


CWidcommStack::~CWidcommStack()
{
	WIDCOMMSDK_ShutDown();
}


void CWidcommStack::OnDeviceResponded(BD_ADDR bda, DEV_CLASS devClass, BD_NAME bdName, BOOL bConnected)
{
	BLUETOOTH_DEVICE_INFO*		pBtDevInfo;

	//
	// Allocate the device info structure on the local heap
	//
	pBtDevInfo = (BLUETOOTH_DEVICE_INFO*)LocalAlloc(LPTR, sizeof(BLUETOOTH_DEVICE_INFO));

	if(!pBtDevInfo)
		return;

	//
	// Prepare the device information structure
	//
	pBtDevInfo->dwSize = sizeof(BLUETOOTH_DEVICE_INFO);

	// Set the device address
	pBtDevInfo->Address.ullLong =	(ULONGLONG)bda[0] << 40 |
									(ULONGLONG)bda[1] << 32 |
									(ULONGLONG)bda[2] << 24 |
									(ULONGLONG)bda[3] << 16 |
									(ULONGLONG)bda[4] <<  8 |
									(ULONGLONG)bda[5];

	//memcpy(pBtDevInfo->Address.rgBytes, bda, sizeof(UINT8) * BD_ADDR_LEN);

	// Set the device class
	pBtDevInfo->ulClassofDevice = ((ULONG)devClass[0] << 16) + ((ULONG)devClass[1] << 8) + devClass[2];

	MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, (const char*)bdName, BD_NAME_LEN, pBtDevInfo->szName, BD_NAME_LEN);

	if(m_deviceResponded != NULL)
		m_deviceResponded(pBtDevInfo);

	//
	// Here, we can also use the asynchronous PostMessage because the
	// device info structures are allocated on the heap and are released
	// by the consuming applciation
	//
	if(m_hWnd != NULL)
		PostMessage(m_hWnd, m_nMsg, 0, (LPARAM)pBtDevInfo);
}


void CWidcommStack::OnInquiryComplete(BOOL bSuccess, short nResponses)
{
	if(m_inquiryComplete != NULL)
		m_inquiryComplete(bSuccess, nResponses);

	if(m_hWnd != NULL)
		PostMessage(m_hWnd, m_nMsg, 1, MAKELPARAM(bSuccess, nResponses));
}


void CWidcommStack::OnDiscoveryComplete()
{
	if(m_discoveryComplete != NULL)
		m_discoveryComplete();

	if(m_hWnd != NULL)
		PostMessage(m_hWnd, m_nMsg, 0, 0);
}
