// wcbts.cpp : Defines the entry point for the DLL application.
//

#include "stdafx.h"
#include "wcbts.h"
#include "WidcommStack.h"
#include "WidcommSppServer.h"


HMODULE g_hModule = NULL;


BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}


WCBTS_API BOOL WidcommStackPresent()
{
	HMODULE hModule;

	hModule = LoadLibrary(L"BtSdkCE30.dll");
	if(hModule != NULL)
	{
		FreeLibrary(hModule);
		return 3;
	}

	hModule = LoadLibrary(L"BtSdkCE50.dll");
	if(hModule != NULL)
	{
		FreeLibrary(hModule);
		return 5;
	}

	return 0;
}


WCBTS_API void* CreateStack()
{
	if(g_hModule == NULL)
		g_hModule = LoadLibrary(L"wcbts.dll");

	return new CWidcommStack();
}


WCBTS_API void DeleteStack(void *pStack)
{
	CWidcommStack*	pWcStack = (CWidcommStack*)pStack;

	if(pWcStack)
		delete pWcStack;

	if(g_hModule != NULL)
	{
		FreeLibrary(g_hModule);
		g_hModule = NULL;
	}
}


WCBTS_API void StackNotifyWindow(void *pStack, HWND hWnd, UINT nMsg)
{
	CWidcommStack*	pWcStack = (CWidcommStack*)pStack;

	if(pWcStack)
		pWcStack->NotifyWindow(hWnd, nMsg);
}


WCBTS_API BOOL StackStartInquiry(void *pStack)
{
	CWidcommStack*	pWcStack = (CWidcommStack*)pStack;

	if(pWcStack)
		return pWcStack->StartInquiry();
	return FALSE;
}


WCBTS_API void StackStopInquiry(void *pStack)
{
	CWidcommStack*	pWcStack = (CWidcommStack*)pStack;

	if(pWcStack)
		pWcStack->StopInquiry();
}


WCBTS_API BOOL StackStartDiscovery(void *pStack, ULONGLONG address, GUID *pGuid)
{
	CWidcommStack*	pWcStack = (CWidcommStack*)pStack;

	if(pWcStack)
	{
		BD_ADDR bda;

		bda[0] = ((address >> 40) & 0xff);
		bda[1] = ((address >> 32) & 0xff);
		bda[2] = ((address >> 24) & 0xff);
		bda[3] = ((address >> 16) & 0xff);
		bda[4] = ((address >>  8) & 0xff);
		bda[5] = ( address        & 0xff);

		pWcStack->SetBdAddr(bda);

		return pWcStack->StartDiscovery(bda, pGuid);
	}
	return FALSE;
}


WCBTS_API void* StackGetDiscovery(void *pStack, ULONGLONG address, GUID *pGuid)
{
	CWidcommStack*	pWcStack = (CWidcommStack*)pStack;

	if(pWcStack)
	{
		BD_ADDR		bda;
		SdpRecs*	pRecs;

		bda[0] = ((address >> 40) & 0xff);
		bda[1] = ((address >> 32) & 0xff);
		bda[2] = ((address >> 24) & 0xff);
		bda[3] = ((address >> 16) & 0xff);
		bda[4] = ((address >>  8) & 0xff);
		bda[5] = ( address        & 0xff);

		pRecs = new SdpRecs;
		if(pRecs)
		{
			pRecs->m_nSdpRecs = pWcStack->ReadDiscoveryRecords(bda, 32, pRecs->m_pSdpRecs, pGuid);

			return pRecs;
		}
	}

	return NULL;
}


//---------------------------------------------------------------------------
//
// Discovery API
//
//---------------------------------------------------------------------------

WCBTS_API void DeleteDiscovery(void *pDisc)
{
	SdpRecs* pSdpRecs = (SdpRecs*)pDisc;

	delete pSdpRecs;
}


WCBTS_API int DiscoveryGetSdpRecCount(void *pDisc)
{
	SdpRecs* pSdpRecs = (SdpRecs*)pDisc;

	if(pSdpRecs)
		return pSdpRecs->m_nSdpRecs;
	return 0;
}


WCBTS_API void* DiscoveryGetSdpRec(void *pDisc, int iRecord)
{
	SdpRecs* pSdpRecs = (SdpRecs*)pDisc;

	if(pSdpRecs)
		return &pSdpRecs->m_pSdpRecs[iRecord];
	return NULL;
}


//---------------------------------------------------------------------------
//
// SdpRecord API
//
//---------------------------------------------------------------------------

WCBTS_API BOOL SdpRecFindRFCommChannel(void *pSdp, ULONG *pChannel)
{
	CSdpDiscoveryRec* pRec = (CSdpDiscoveryRec*)pSdp;

	if(pRec)
	{
		UINT8	scn;

		if(pRec->FindRFCommScn(&scn))
		{
			*pChannel = scn;
			return TRUE;
		}
	}
	return FALSE;
}


WCBTS_API BOOL SdpRecFindServiceName(void *pSdp, TCHAR *pszName, int nNameLen)
{
	CSdpDiscoveryRec* pRec = (CSdpDiscoveryRec*)pSdp;

	if(pRec)
	{
		memset(pszName, 0, sizeof(TCHAR) * nNameLen);

		wcsncpy(pszName, pRec->m_service_name, nNameLen);

		return TRUE;
	}
	return FALSE;
}


WCBTS_API BOOL SdpRecFindServiceClass(void *pSdp, GUID *pGuid)
{
	CSdpDiscoveryRec* pRec = (CSdpDiscoveryRec*)pSdp;

	if(pRec)
	{
		memcpy(pGuid, &pRec->m_service_guid, sizeof(GUID));

		return TRUE;
	}
	return FALSE;
}


//---------------------------------------------------------------------------
//
// SppServer API
//
//---------------------------------------------------------------------------

WCBTS_API void* CreateSppServer(void * /*pStack*/)
{
	return new CWidcommSppServer();
}


WCBTS_API void DeleteSppServer(void *pServer)
{
	CWidcommSppServer* pSppServer = (CWidcommSppServer*)pServer;

	if(pSppServer)
		delete pSppServer;
}


WCBTS_API BOOL SppServerStart(void *pServer, LPCTSTR pszServiceName)
{
	CWidcommSppServer* pSppServer = (CWidcommSppServer*)pServer;

	if(pSppServer)
	{
		CSppServer::SPP_SERVER_RETURN_CODE	nRet;

		nRet = pSppServer->CreateConnection((BT_CHAR *)pszServiceName);

		return nRet == CSppServer::SUCCESS;
	}
	return FALSE;
}


WCBTS_API BOOL SppServerStop(void *pServer)
{
	CWidcommSppServer* pSppServer = (CWidcommSppServer*)pServer;

	if(pSppServer)
	{
		CSppServer::SPP_SERVER_RETURN_CODE	nRet;

		nRet = pSppServer->RemoveConnection();

		return nRet == CSppServer::SUCCESS;
	}
	return FALSE;
}

