#ifdef WCBTS_EXPORTS
#define WCBTS_API __declspec(dllexport)
#else
#define WCBTS_API __declspec(dllimport)
#endif


extern "C" {


#define BLUETOOTH_MAX_NAME_SIZE 248

typedef ULONGLONG BTH_ADDR;

typedef struct _BLUETOOTH_ADDRESS
{
	union
	{
		BTH_ADDR	ullLong;
		BYTE		rgBytes[6];  
	};
} BLUETOOTH_ADDRESS;


typedef struct _BLUETOOTH_DEVICE_INFO
{
	DWORD				dwSize;
	BLUETOOTH_ADDRESS	Address;
	ULONG				ulClassofDevice;
	BOOL				fConnected;
	BOOL				fRemembered;
	BOOL				fAuthenticated;
	SYSTEMTIME			stLastSeen;
	SYSTEMTIME			stLastUsed;
	WCHAR				szName[BLUETOOTH_MAX_NAME_SIZE];
} BLUETOOTH_DEVICE_INFO;


typedef struct _BLUETOOTH_SPP_INFO
{
	BLUETOOTH_ADDRESS	Address;
	ULONG				ulClassofDevice;
	BOOL				fConnected;
	short				nPort;
	WCHAR				szName[BLUETOOTH_MAX_NAME_SIZE];
} BLUETOOTH_SPP_INFO;


typedef void (*DeviceRespondedCallback)(BLUETOOTH_DEVICE_INFO *pBtDevInfo);
typedef void (*InquiryCompleteCallback)(BOOL, short);
typedef void (*DiscoveryCompleteCallback)();
typedef void (*SppServerCallback)(BLUETOOTH_SPP_INFO *pBtSppInfo);


//---------------------------------------------------------------------------
//
// Stack API
//
//---------------------------------------------------------------------------

WCBTS_API	int		WidcommStackPresent();

WCBTS_API	void*	CreateStack();
WCBTS_API	void	DeleteStack			(void *pStack);
WCBTS_API	void	StackNotifyWindow	(void *pStack, HWND hWnd, UINT nMsg);
WCBTS_API	BOOL	StackStartInquiry	(void *pStack);
WCBTS_API	void	StackStopInquiry	(void *pStack);
WCBTS_API	BOOL	StackStartDiscovery	(void *pStack, ULONGLONG address, GUID *pGuid);

WCBTS_API	void*	StackGetDiscovery	(void *pStack, ULONGLONG address, GUID *pGuid);


//---------------------------------------------------------------------------
//
// Discovery API
//
//---------------------------------------------------------------------------

WCBTS_API	int		DiscoveryGetSdpRecCount	(void *pDisc);
WCBTS_API	void*	DiscoveryGetSdpRec		(void *pDisc, int iRecord);
WCBTS_API	void	DeleteDiscovery			(void *pDisc);


//---------------------------------------------------------------------------
//
// SdpRecord API
//
//---------------------------------------------------------------------------

WCBTS_API	BOOL	SdpRecFindRFCommChannel	(void *pSdp, ULONG *pChannel);
WCBTS_API	BOOL	SdpRecFindServiceName	(void *pSdp, TCHAR *pszName, int nNameLen);
WCBTS_API	BOOL	SdpRecFindServiceClass	(void *pSdp, GUID *pGuid);


//---------------------------------------------------------------------------
//
// SppServer API
//
//---------------------------------------------------------------------------

WCBTS_API	void*	CreateSppServer			(void *pStack);
WCBTS_API	void	DeleteSppServer			(void *pServer);
WCBTS_API	BOOL	SppServerStart			(void *pServer, LPCTSTR pszServiceName);
WCBTS_API	BOOL	SppServerStop			(void *pServer);
WCBTS_API	void	SppServerNotifyWindow	(void *pServer, HWND hWnd, UINT nMsg);


//---------------------------------------------------------------------------
//
// SppClient API
//
//---------------------------------------------------------------------------

WCBTS_API	void*	CreateSppClient			(void *pStack);
WCBTS_API	void	DeleteSppClient			(void *pClient);
WCBTS_API	BOOL	SppClientStart			(void *pClient, ULONGLONG btAddr, LPCTSTR pszServiceName);
WCBTS_API	void	SppClientStop			(void *pClient);
WCBTS_API	void	SppClientNotifyWindow	(void *pClient, HWND hWnd, UINT nMsg);

};	// extern "C"
